/* ----------------------------------------------------------------
 *
 * andorTCP.h
 *
 * ---------------------------------------------------------------- */

#include <pthread.h>

#ifdef MACOSX
#include <sys/types.h>
#endif

/* ---------------------------------------------------------------- */

#define AT_ERR_HARDWARE_OVERFLOW  100

enum andor_errors_enum { E_andor_error = 1+AT_ERR_HARDWARE_OVERFLOW,
                         E_andor_not_connected,
                         E_andor_setpar,
                         E_andor_last };

#define ANDOR_NBUFS   3

typedef struct andor_frame_tag {
  double timeStamp;
  u_int seqNumber;
  u_short *data;
  int w,h;
  volatile int wlock,rlock;
} AndorFrame;

typedef struct andor_struct_tag {
  char   host[128];
  int    port,handle; // TODO sock
  volatile int err;
  char   serverVersion[64],serialNumber[32];
  u_int  cookie;
  float  tempSensor;
  char   tempStatus[32];
  int    aoiW,aoiH;
  double expTime;  
  double fps;
  volatile int rolling;
  pthread_mutex_t ioLock,frameLock;
  u_int seqNumber;
  AndorFrame frames[ANDOR_NBUFS];
  pthread_t tid;
  volatile int stop_flag;
} AndorStruct;

/* ---------------------------------------------------------------- */

AndorStruct* andor_create     (const char*,int);

int  andor_connect    (AndorStruct*);
int  andor_open       (AndorStruct*);
int  andor_setup      (AndorStruct*,int,int);
int  andor_disconnect (AndorStruct*);
int  andor_close      (AndorStruct*);
void andor_free       (AndorStruct*);

int andor_temperature (AndorStruct*);
int andor_tempcon     (AndorStruct*,int);
int andor_exptime     (AndorStruct*,double);
int andor_rolling     (AndorStruct*,int);

int andor_cycle_start (AndorStruct*);
int andor_cycle_stop  (AndorStruct*);

AndorFrame* andor_frame4writing(AndorStruct*,u_int);
AndorFrame* andor_frame4reading(AndorStruct*,u_int);
void        andor_frame_release(AndorStruct*,AndorFrame*);

int    andor_server(AndorStruct*,const char*,char*);

int    andor_sim   (AndorStruct*,const char*,char*);

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */

