/* ----------------------------------------------------------------
 *
 * andorUSB.h
 *
 * ---------------------------------------------------------------- */

#include <pthread.h>

#ifdef MACOSX
#include <sys/types.h>
#endif

#ifndef SIM_ONLY
#include "atcore.h"                    /* /usr/local/include/atcore.h */
#else
#define AT_SUCCESS 0
// #define AT_ERR_NOTINITIALISED 1
// #define AT_ERR_NOTIMPLEMENTED 2
// #define AT_ERR_READONLY 3
// #define AT_ERR_NOTREADABLE 4
// #define AT_ERR_NOTWRITABLE 5
// #define AT_ERR_OUTOFRANGE 6
// #define AT_ERR_INDEXNOTAVAILABLE 7
// #define AT_ERR_INDEXNOTIMPLEMENTED 8
// #define AT_ERR_EXCEEDEDMAXSTRINGLENGTH 9
// #define AT_ERR_CONNECTION 10
// #define AT_ERR_NODATA 11
// #define AT_ERR_INVALIDHANDLE 12
#define AT_ERR_TIMEDOUT 13
// #define AT_ERR_BUFFERFULL 14
// #define AT_ERR_INVALIDSIZE 15
// #define AT_ERR_INVALIDALIGNMENT 16
#define AT_ERR_COMM 17
// #define AT_ERR_STRINGNOTAVAILABLE 18
// #define AT_ERR_STRINGNOTIMPLEMENTED 19
// #define AT_ERR_NULL_FEATURE 20
// #define AT_ERR_NULL_HANDLE 21
// #define AT_ERR_NULL_IMPLEMENTED_VAR 22
// #define AT_ERR_NULL_READABLE_VAR 23
// #define AT_ERR_NULL_READONLY_VAR 24
// #define AT_ERR_NULL_WRITABLE_VAR 25
// #define AT_ERR_NULL_MINVALUE 26
// #define AT_ERR_NULL_MAXVALUE 27
// #define AT_ERR_NULL_VALUE 28
// #define AT_ERR_NULL_STRING 29
// #define AT_ERR_NULL_COUNT_VAR 30
// #define AT_ERR_NULL_ISAVAILABLE_VAR 31
// #define AT_ERR_NULL_MAXSTRINGLENGTH 32
// #define AT_ERR_NULL_EVCALLBACK 33
// #define AT_ERR_NULL_QUEUE_PTR 34
// #define AT_ERR_NULL_WAIT_PTR 35
// #define AT_ERR_NULL_PTRSIZE 36
// #define AT_ERR_NOMEMORY 37
// #define AT_ERR_DEVICEINUSE 38
// #define AT_ERR_DEVICENOTFOUND 39
#define AT_ERR_HARDWARE_OVERFLOW  100
typedef int AT_H;
typedef int AT_BOOL;
typedef long long AT_64;
typedef unsigned char AT_U8;
typedef wchar_t AT_WC;
#define AT_TRUE 1
#define AT_FALSE 0
#endif

/* ---------------------------------------------------------------- */

enum andor_errors_enum { E_andor_error = 1+AT_ERR_HARDWARE_OVERFLOW,
                         E_andor_not_connected,
                         E_andor_not_present,
                         E_andor_setpar,
                         E_andor_last };

#define ANDOR_NBUFS   3

typedef struct andor_frame_tag {
  double timeStamp;
  u_int seqNumber;
  u_short *data;
  int w,h;
  volatile int wlock,rlock;
} AndorFrame;

typedef struct andor_struct_tag {
  int    id;
  AT_H   handle;
  int    initialized;
  volatile int err;
  char   model[32],serialNumber[32],name[32],firmware[32];
  float  tempSensor;
  char   tempStatus[32];
  int    bias;
  int    sensorW,sensorH;
  int    aoiW,aoiH,binH,binV;
  double expTime;  
  double readoutTime;          /* NOTE: does NOT depend on binning */
  double fps;
  pthread_mutex_t ioLock,frameLock;
  u_int seqNumber;             /* last frame number read from AT_() */
  u_char *data_1,*data_2;      /* IDEA test triple buffer */
  AndorFrame frames[ANDOR_NBUFS];
  pthread_t tid;
  volatile int stop_flag;
} AndorStruct;

/* ---------------------------------------------------------------- */

AndorStruct* andor_create(int);
int          andor_open  (AndorStruct*);
int          andor_init  (AndorStruct*);
int          andor_close (AndorStruct*);
int          andor_free  (AndorStruct*);

int andor_aoi         (AndorStruct*,int,int);
int andor_temperature (AndorStruct*);
int andor_tempcon     (AndorStruct*,int value);
int andor_exptime     (AndorStruct*,double exptime);
int andor_binningX    (AndorStruct*,int value);
int andor_binningY    (AndorStruct*,int value);
int andor_binningXY   (AndorStruct*,int value);

int andor_cycle_start (AndorStruct*);
int andor_cycle_stop  (AndorStruct*);

AndorFrame* andor_frame4writing(AndorStruct*,u_int);
AndorFrame* andor_frame4reading(AndorStruct*,u_int);
void        andor_frame_release(AndorStruct*,AndorFrame*);

int  andor_sim(AndorStruct*,const char*,char*);

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */

