/* --------------------------------------------------------------------- 
 *
 * andorcmd.c / tcpsend.c         
 *
 * send TCP commands to AndorServer
 * 
 * v0.01 1998-04-15  Christoph C. Birk, birk@ociw.edu
 * v1.00 2003-06-27  [CCB]
 * v2.00 2018-05-18  Andor version [CCB]
 *
 * cc tcpsend.c -lnsl -lsocket
 *
 * --------------------------------------------------------------------- */

/* DEFINEs ------------------------------------------------------------- */

#define DEBUG           1               /* debug level */

/* INCLUDEs ------------------------------------------------------------ */

#include <stdlib.h>                     /* exit() */
#include <stdio.h>                      /* fprintf() */
#include <string.h>                     /* strcpy() */
#include <unistd.h>                     /* close() */
#include <errno.h>

#include <sys/socket.h>                 /* socket() */
#include <netdb.h>                      /* getprotobyname() */
#include <netinet/in.h>                 /* sockaddr_in */
#include <fcntl.h>

#include "andor.h"
#include "utils.h"

/* GLOBALs ------------------------------------------------------------- */

/* EXTERNs ------------------------------------------------------------- */

/* Function(s) --------------------------------------------------------- */

int main(int argc,char **argv)
{
  int     port=SERVER_PORT,timeout=5;
  int     i,sock,j,r,loop=1,wait=500;
  char    host[128]="localhost",command[256]="";
  char    buffer[1024];
  fd_set  rfd; 
  struct  timeval tout;
  struct  protoent     *proto;
  struct  hostent      *hostp;
  struct  sockaddr_in  saddr;

  (void)gethostname(host,sizeof(host));

  { extern char *optarg;               /* parse command line */
    extern int opterr,optopt,optind; opterr=0;
    while ((i=getopt(argc,argv,"i:h:l:p:t:w:")) != EOF) {
      switch (i) {
      case 'i':
        port = port+atoi(optarg);
        break;
      case 'h':
        strcpy(host,optarg);
        break;
      case 'l': 
        loop = atoi(optarg);
        break;
      case 'p':
        port = atoi(optarg);
        break;
      case 't':
        timeout = atoi(optarg);
        break;
      case 'w':
        wait = atoi(optarg);
        break;
      }
    }
    if (optind < argc) strcpy(command,argv[optind++]);
    while (optind < argc) {
      strcat(command," "); strcat(command,argv[optind++]);
    }
  }
  if (port == 0) { fprintf(stderr,"%s: illegal port\n",argv[0]); exit(1); }
  if (*command == '\0') { fprintf(stderr,"%s: no command\n",argv[0]); exit(1); }
#if (DEBUG > 1)
  fprintf(stderr,"cr=%d\n",cr);
  fprintf(stderr,"host=%s, port=%d, command='%s'\n",host,port,command);
#endif

  proto = getprotobyname("tcp");
  if (proto == NULL) {
    fprintf(stderr,"%s: getprotobyname(tcp) failed\n",argv[0]);
    exit(2);
  }
  hostp = gethostbyname(host);
  if (hostp == NULL) {
    fprintf(stderr,"%s: cannot resolve host '%s'\n",argv[0],host);
    exit(3);
  }
#if (DEBUG > 0)
  sprintf(buffer,"host='%s' (%d.%d.%d.%d)",hostp->h_name,
    *(u_char*)(hostp->h_addr_list[0]+0),
    *(u_char*)(hostp->h_addr_list[0]+1),
    *(u_char*)(hostp->h_addr_list[0]+2),
    *(u_char*)(hostp->h_addr_list[0]+3)
  );
  fprintf(stderr,"%s - %s\n",argv[0],buffer);
#endif

  if ((sock = socket(AF_INET, SOCK_STREAM, proto->p_proto)) == -1) {
    fprintf(stderr,"%s - %s\n",argv[0],"cannot create stream socket");
    exit(1);
  }

  memset((void*)&saddr,0,sizeof(saddr));
  saddr.sin_family = AF_INET;
  memcpy(&saddr.sin_addr, hostp->h_addr_list[0], hostp->h_length);
  saddr.sin_port = htons(port);
  (void)fcntl(sock,F_SETFD,FD_CLOEXEC);        /* close-on-exec flag */

  if (connect(sock,(struct sockaddr *)&saddr,sizeof(saddr)) < 0) {
    fprintf(stderr,"error connecting to stream socket, errno=%d (%s)\n",
            errno,strerror(errno)); 
    (void)close(sock);
    return(-1);
  }

  strcat(command,"\n");                /* <LF> terminated */

  while (loop > 0) {

  if (send(sock,command,strlen(command),0) < 0) {
    fprintf(stderr,"error writing on stream socket\n"); 
    (void)close(sock);
    return(-1);
  } 

  for (j=0; j<sizeof(buffer)-1; j++) {
    do {
      FD_ZERO(&rfd); FD_SET(sock,&rfd);
      tout.tv_sec = timeout; tout.tv_usec = 0;
      r = select(sock+1,&rfd,NULL,NULL,&tout);
      if (FD_ISSET(sock,&rfd)) break;
    } while (r > 0);
    if (r <= 0) break;
    r = recv(sock,&buffer[j],1,0);
    if (r <= 0) break;
    if (buffer[j] == '\n') break;
  }
  buffer[j] = '\0';                    /* terminate */
  printf("%s\n",buffer);

  if (!strcmp(command,"next\n")) { u_int seq; int dim,i,n; double tst;
    if (!strstr(buffer,"-E")) {
      sscanf(buffer,"%u %d %lf",&seq,&dim,&tst);
      int nbytes = dim*dim*sizeof(u_short);
      char *data = (char*)malloc(nbytes);
      double t1 = walltime(0);
      for (i=0,n=0; i<nbytes/42; i++) {
        ssize_t r = recv(sock,data+n,nbytes-n,0);
        if (r <= 0) break;
        printf(".");
        n += r; if (n >= nbytes) break;
      }
      double t2 = walltime(0);
      printf("\n%d bytes (%.1f MB/s)\n",n,(float)n/(1.0e6*(t2-t1)));
      free((void*)data);
    }
  }

    loop--; if (loop) msleep(wait);
  } //endwhile(loop)
  (void)close(sock);

  return 0;
}

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */

